@@ -6,11 +6,17 @@ import subprocess
 import hvac
 import requests
 import tldextract
-import zx
 from loguru import logger
 from tenacity import retry, stop_after_attempt, wait_random
 
 
+def rmdir(dir_location):
+    if os.path.isdir(dir_location):
+        logger.info(f"Removing existing config dir {dir_location}")
+        shutil.rmtree(dir_location)
+        logger.info(f"Removed existing config dir {dir_location}")
+
+
 class RundeckJobs(object):
     def __init__(
             self,
@@ -89,10 +95,7 @@ class RundeckJobs(object):
         pg_backend_conn_str = f"postgres://{pg_user}:{pg_password}@{pg_ip}:{pg_port}/{pg_db}"
 
         config_dir_location = f'/tmp/{fqdn}'
-        if os.path.isdir(config_dir_location):
-            logger.info(f"Removing existing config dir {config_dir_location}")
-            shutil.rmtree(config_dir_location)
-            logger.info(f"Removed existing config dir {config_dir_location}")
+        rmdir(config_dir_location)
 
         backend_config_file_location = f"/tmp/{fqdn}.backend.tfvars"
         backend_config_file_content = f'''conn_str = "{pg_backend_conn_str}"'''
@@ -121,32 +124,19 @@ class RundeckJobs(object):
         os.environ["TF_IN_AUTOMATION"] = "true"
 
         shutil.copytree(f"{terraform_code_dir}/", f"{config_dir_location}/")
-        # zx.run_shell_print(
-        #     f"cd {config_dir_location} && \
-        #     terraform init -input=false -reconfigure -force-copy -backend-config {backend_config_file_location}")
         self.run_command(["terraform", "init", "-input=false", "-reconfigure", "-force-copy",
                           f"-backend-config", backend_config_file_location],
                          cwd=config_dir_location, check=True)
         try:
-            # zx.run_shell_print(
-            #     f"cd {config_dir_location} && \
-            #     terraform workspace select {fqdn}.{deployment_environment}.{datacenter}")
             subprocess.run(["terraform", "workspace", "select", f"{fqdn}.{deployment_environment}.{datacenter}"],
                            cwd=config_dir_location, check=True)
         except subprocess.CalledProcessError:
-            # zx.run_shell_print(
-            #     f"cd {config_dir_location} && \
-            #     terraform workspace new {fqdn}.{deployment_environment}.{datacenter}")
             subprocess.run(["terraform", "workspace", "new", f"{fqdn}.{deployment_environment}.{datacenter}"],
                            cwd=config_dir_location, check=True)
 
         plan_file_location = f"{config_dir_location}/config.plan"
 
         if deployment_action == "apply":
-            # zx.run_shell_print(
-            #     f"cd {config_dir_location} && \
-            #     terraform plan -out {plan_file_location} -input=false -var-file {vars_file_location} && \
-            #     terraform apply -input=false -auto-approve {plan_file_location}")
             self.run_command(
                 ["terraform", "plan", "-out", plan_file_location,
                  "-input=false", "-var-file", vars_file_location],
@@ -154,12 +144,6 @@ class RundeckJobs(object):
             self.run_command(["terraform", "apply", "-input=false", "-auto-approve", plan_file_location],
                              cwd=config_dir_location, check=True)
         elif deployment_action == "destroy":
-            # zx.run_shell_print(
-            #     f"cd {config_dir_location} && \
-            #     terraform plan -destroy -out {plan_file_location} -input=false -var-file {vars_file_location} && \
-            #     terraform apply -input=false -auto-approve {plan_file_location} && \
-            #     terraform workspace select default && \
-            #     terraform workspace delete {fqdn}.{deployment_environment}.{datacenter}")
             self.run_command(["terraform", "plan", "-destroy", "-out", plan_file_location, "-input=false", "-var-file",
                               vars_file_location], cwd=config_dir_location, check=True)
             self.run_command(["terraform", "apply", "-input=false", "-auto-approve", plan_file_location],
@@ -229,7 +213,6 @@ class RundeckJobs(object):
             url='https://vault.zadapps.info',
 
         )
-        # vault_client.auth_userpass(vault_login_username, vault_login_password)
         vault_client.login(
             "/v1/auth/{0}/login/{1}".format("userpass", vault_login_username),
             json={"password": vault_login_password},
@@ -238,10 +221,7 @@ class RundeckJobs(object):
             path=f'tenant/devops/{project_name}/hetzner')['data']['data']['api_token']
 
         config_dir_location = f'/tmp/{fqdn}'
-        if os.path.isdir(config_dir_location):
-            logger.info(f"Removing existing config dir {config_dir_location}")
-            shutil.rmtree(config_dir_location)
-            logger.info(f"Removed existing config dir {config_dir_location}")
+        rmdir(config_dir_location)
 
         backend_config_file_location = f"/tmp/{fqdn}.backend.tfvars"
         backend_config_file_content = '''
@@ -266,11 +246,6 @@ class RundeckJobs(object):
         os.environ["TF_IN_AUTOMATION"] = "true"
 
         shutil.copytree(f"{terraform_code_dir}/", f"{config_dir_location}/")
-        # zx.run_shell_print(
-        #     f"cd {config_dir_location} && \
-        #     terraform init -input=false -reconfigure -force-copy \
-        #     -backend-config {backend_config_file_location}"
-        # )
         self.run_command(["terraform", "init", "-input=false", "-reconfigure", "-force-copy", "-backend-config",
                           backend_config_file_location], cwd=config_dir_location,
                          check=True)
@@ -307,46 +282,11 @@ class RundeckJobs(object):
             f.write(vars_file_content)
 
         if deployment_action == "apply":
-            # zx.run_shell_print(f"cd {config_dir_location} && \
-            # terraform apply -input=false -auto-approve")
             self.run_command(["terraform", "apply", "-input=false", "-auto-approve"], cwd=config_dir_location,
                              check=True)
             if deployment_environment == "testing":
                 deployment_action = "destroy"
                 destroy_delay = "3d"
-                # zx.run_shell_print(
-                #     f"rd run --project main --job Datacenter/Hetzner/Cloud/Deploy --delay {destroy_delay} -- \
-                #      -deployment_action {deployment_action} \
-                #      -fqdn {fqdn} \
-                #      -project_name {project_name} \
-                #      -sld {sld} \
-                #      -webserver_netdata_fqdn {webserver_netdata_fqdn} \
-                #      -mariadb_server_netdata_fqdn {mariadb_server_netdata_fqdn} \
-                #      -webserver_fqdn {webserver_fqdn} \
-                #      -mariadb_server_fqdn {mariadb_server_fqdn} \
-                #      -deployment_environment {deployment_environment} \
-                #      -setup_wordpress {setup_wordpress} \
-                #      -wordpress_site_title {wordpress_site_title} \
-                #      -with_wordpress_lifter_lms {with_wordpress_lifter_lms} \
-                #      -wordpress_lms_config_repo {wordpress_lms_config_repo} \
-                #      -wordpress_lms_config_repo_script_dir {wordpress_lms_config_repo_script_dir} \
-                #      -wordpress_lms_config_repo_script_name {wordpress_lms_config_repo_script_name} \
-                #      -with_lifter_lms_loadtest_course {with_lifter_lms_loadtest_course} \
-                #      -with_internal_mariadb {with_internal_mariadb} \
-                #      -hcloud_token {hcloud_token} \
-                #      -vault_login_username {vault_login_username} \
-                #      -vault_login_password {vault_login_password} \
-                #      -atlas_mongo_public_key {atlas_mongo_public_key} \
-                #      -atlas_mongo_private_key {atlas_mongo_private_key} \
-                #      -web_server_image {web_server_image} \
-                #      -mariadb_server_image {mariadb_server_image} \
-                #      -web_server_type {web_server_type} \
-                #      -mariadb_server_type {mariadb_server_type} \
-                #      -jira_issue_key {jira_issue_key} \
-                #      -tag_names {os.environ.get('RD_OPTION_TAG_NAMES')} \
-                #      -auto_apply {auto_apply} \
-                #      -execution_mode {execution_mode}"
-                # )
                 self.run_command(
                     ["rd", "run", "--project", "main", "--job", "Datacenter/Hetzner/Cloud/Deploy", "--delay",
                      destroy_delay, "--",
@@ -383,8 +323,6 @@ class RundeckJobs(object):
                     cwd=config_dir_location, check=True)
 
         elif deployment_action == "destroy":
-            # zx.run_shell_print(f"cd {config_dir_location} && \
-            # terraform destroy -input=false -auto-approve")
             self.run_command(["terraform", "destroy", "-input=false", "-auto-approve"], cwd=config_dir_location,
                              check=True)
             pg_creds = vault_client.secrets.kv.read_secret_version(
@@ -402,3 +340,94 @@ class RundeckJobs(object):
                                                'RD_OPTION_WITH_VCS_REPO'),
                                            execution_mode=execution_mode, deployment_environment=deployment_environment,
                                            deployment_action=deployment_action)
+
+    def nextjs_build(
+            self,
+            pg_user: str = os.environ.get(
+                'RD_OPTION_TERRAFORM_PG_BACKEND_USER'),
+            pg_password: str = os.environ.get(
+                'RD_OPTION_TERRAFORM_PG_BACKEND_PASSWORD'),
+            pg_ip: str = os.environ.get('RD_OPTION_TERRAFORM_PG_BACKEND_IP'),
+            pg_port: str = os.environ.get(
+                'RD_OPTION_TERRAFORM_PG_BACKEND_PORT'),
+            pg_db: str = os.environ.get('RD_OPTION_TERRAFORM_PG_BACKEND_DB'),
+            fqdn: str = os.environ.get("RD_OPTION_FQDN"),
+            organization: str = os.environ.get('RD_OPTION_ORGANIZATION'),
+            deployment_action: str = os.environ.get(
+                'RD_OPTION_DEPLOYMENT_ACTION'),
+            git_domain: str = os.environ.get('RD_OPTION_GIT_DOMAIN'),
+            git_reponame: str = os.environ.get('RD_OPTION_GIT_REPONAME'),
+            git_branch: str = os.environ.get('RD_OPTION_GIT_BRANCH'),
+            workspace_name: str = os.environ.get('RD_OPTION_WORKSPACE_NAME'),
+            app_code_dir: str = os.environ.get('RD_OPTION_APP_CODE_DIR'),
+            vault_login_username: str = os.environ.get(
+                'RD_OPTION_VAULT_LOGIN_USERNAME'),
+            vault_login_password: str = os.environ.get(
+                'RD_OPTION_VAULT_LOGIN_PASSWORD'),
+            terraform_code_dir: str = f"{os.path.expanduser('~')}/codebase/terraform/rootmodule/nextjs_build",
+    ):
+        pg_backend_conn_str = f"postgres://{pg_user}:{pg_password}@{pg_ip}:{pg_port}/{pg_db}"
+
+        config_dir_location = f'/tmp/{fqdn}.build'
+        rmdir(config_dir_location)
+
+        backend_config_file_location = f"/tmp/{fqdn}.build.backend.tfvars"
+        backend_config_file_content = f'''conn_str = "{pg_backend_conn_str}"'''
+        with open(backend_config_file_location, "w") as f:
+            f.write(backend_config_file_content)
+
+        fqdn_parts = tldextract.extract(fqdn)
+        sld = f"{fqdn_parts.domain}.{fqdn_parts.suffix}"
+        vars_file_location = f"/tmp/{fqdn}.build.tfvars"
+        vars_file_content = f"""
+        fqdn = "{fqdn}"
+        sld = "{sld}"
+        project_name = "{organization}"
+        deployment_action = "{deployment_action}"
+        git_domain = "{git_domain}"
+        git_reponame = "{git_reponame}"
+        git_branch = "{git_branch}"
+        workspace_name = "{workspace_name}"
+        app_code_dir = "{app_code_dir}"
+        vault_login_username = "{vault_login_username}"
+        vault_login_password = "{vault_login_password}"
+        """
+        with open(vars_file_location, "w") as f:
+            f.write(vars_file_content)
+
+        os.environ["PATH"] += os.pathsep + f"{os.path.expanduser('~')}/bin"
+        os.environ["TF_REGISTRY_DISCOVERY_RETRY"] = "10"
+        os.environ["TF_REGISTRY_CLIENT_TIMEOUT"] = "60"
+        os.environ["TF_IN_AUTOMATION"] = "true"
+        os.environ["TF_PLUGIN_CACHE_DIR="] = "~/.terraform.d/plugin-cache"
+        os.environ["TF_DATA_DIR"] = config_dir_location
+
+        # shutil.copytree(f"{terraform_code_dir}/", f"{config_dir_location}/")
+        self.run_command(["terraform", "init", "-input=false", "-reconfigure", "-force-copy",
+                          f"-backend-config", backend_config_file_location],
+                         cwd=terraform_code_dir, check=True)
+        try:
+            subprocess.run(["terraform", "workspace", "select", f"{fqdn}.build"],
+                           cwd=terraform_code_dir, check=True)
+        except subprocess.CalledProcessError:
+            subprocess.run(["terraform", "workspace", "new", f"{fqdn}.build"],
+                           cwd=terraform_code_dir, check=True)
+
+        plan_file_location = f"{config_dir_location}/config.plan"
+
+        if deployment_action == "apply":
+            self.run_command(
+                ["terraform", "plan", "-out", plan_file_location,
+                 "-input=false", "-var-file", vars_file_location],
+                cwd=terraform_code_dir, check=True)
+            self.run_command(["terraform", "apply", "-input=false", "-auto-approve", plan_file_location],
+                             cwd=terraform_code_dir, check=True)
+        elif deployment_action == "destroy":
+            self.run_command(["terraform", "plan", "-destroy", "-out", plan_file_location, "-input=false", "-var-file",
+                              vars_file_location], cwd=terraform_code_dir, check=True)
+            self.run_command(["terraform", "apply", "-input=false", "-auto-approve", plan_file_location],
+                             cwd=terraform_code_dir, check=True)
+            self.run_command(["terraform", "workspace", "select",
+                              "default"], cwd=terraform_code_dir, check=True)
+            self.run_command(["terraform", "workspace", "delete", f"{fqdn}.build"],
+                             cwd=terraform_code_dir, check=True)
